"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var _this = this;
Object.defineProperty(exports, "__esModule", { value: true });
var index_1 = require("../index");
describe('Tasks', function () {
    test('Can create an empty task', function () {
        var resolve = jest.fn();
        var reject = jest.fn();
        var task = index_1.default.empty();
        task.fork(resolve, reject);
        expect(resolve).not.toHaveBeenCalled();
        expect(reject).not.toHaveBeenCalled();
    });
    test('Can create a task of value', function () {
        var resolve = jest.fn();
        var reject = jest.fn();
        var task = index_1.default.of(true);
        task.fork(resolve, reject);
        expect(resolve).toHaveBeenCalledWith(true);
        expect(reject).not.toHaveBeenCalled();
    });
    test('Can create a rejected task', function () {
        var resolve = jest.fn();
        var reject = jest.fn();
        var task = index_1.default.rejected(true);
        task.fork(resolve, reject);
        expect(resolve).not.toHaveBeenCalled();
        expect(reject).toHaveBeenCalledWith(true);
    });
    test('Can create a task from a promise', function () {
        index_1.default.fromPromise(Promise.resolve('a')).fork(function (x) {
            expect(x).toBe('a');
        });
        index_1.default.fromPromise(Promise.reject('b')).fork(function () { return undefined; }, function (err) {
            expect(err).toBe('b');
        });
        index_1.default.fromPromise(new Promise(function (resolve) { return setTimeout(function () { return resolve('a'); }, 50); })).fork(function (x) {
            expect(x).toBe('a');
        });
    });
    test('Can create a task from a promise creator', function () {
        index_1.default.fromPromise(function () { return Promise.resolve('a'); }).fork(function (x) {
            expect(x).toBe('a');
        });
    });
    test('Can create a promise from a task', function () { return __awaiter(_this, void 0, void 0, function () {
        var task, x;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    task = index_1.default.of('a');
                    return [4 /*yield*/, task.toPromise()];
                case 1:
                    x = _a.sent();
                    expect(x).toBe('a');
                    return [2 /*return*/];
            }
        });
    }); });
    test('Can map a successful task', function () {
        var resolve = jest.fn();
        var reject = jest.fn();
        var task = index_1.default.of('a');
        task.map(function (v) { return v.toUpperCase(); }).fork(resolve, reject);
        expect(resolve).toHaveBeenCalledWith('A');
        expect(reject).not.toHaveBeenCalled();
    });
    test('Does not map a failed task', function () {
        var resolve = jest.fn();
        var reject = jest.fn();
        var task = index_1.default.rejected('a');
        task.map(function (v) { return v.toUpperCase(); }).fork(resolve, reject);
        expect(resolve).not.toHaveBeenCalled();
        expect(reject).toHaveBeenCalledWith('a');
    });
    test('Can map the rejected value of a failed task', function () {
        var resolve = jest.fn();
        var reject = jest.fn();
        var task = index_1.default.rejected('a');
        task.mapRejected(function (v) { return v.toUpperCase(); }).fork(resolve, reject);
        expect(resolve).not.toHaveBeenCalled();
        expect(reject).toHaveBeenCalledWith('A');
    });
    test('Can chain a successful task', function () {
        var resolve = jest.fn();
        var reject = jest.fn();
        var task = index_1.default.of('a');
        task.chain(function (v) { return index_1.default.of(v.toUpperCase()); }).fork(resolve, reject);
        expect(resolve).toHaveBeenCalledWith('A');
        expect(reject).not.toHaveBeenCalled();
    });
    test('Does not chain a failed task', function () {
        var resolve = jest.fn();
        var reject = jest.fn();
        var task = index_1.default.rejected('a');
        task.chain(function (v) { return index_1.default.of(v.toUpperCase()); }).fork(resolve, reject);
        expect(resolve).not.toHaveBeenCalled();
        expect(reject).toHaveBeenCalledWith('a');
    });
    test('Can chain a multiple tasks', function () { return __awaiter(_this, void 0, void 0, function () {
        var addValue, task1, task2, task3;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    addValue = jest.fn(function (x) { return x; });
                    task1 = new index_1.default(function (resolve, _) {
                        setTimeout(function () {
                            resolve(addValue('a'));
                        }, 60);
                    });
                    task2 = new index_1.default(function (resolve, _) {
                        setTimeout(function () {
                            resolve(addValue('b'));
                        }, 40);
                    });
                    task3 = new index_1.default(function (resolve, _) {
                        setTimeout(function () {
                            resolve(addValue('c'));
                        }, 20);
                    });
                    return [4 /*yield*/, task1
                            .chain(function () { return task2; })
                            .chain(function () { return task3; })
                            .toPromise()];
                case 1:
                    _a.sent();
                    expect(addValue.mock.calls).toEqual([['a'], ['b'], ['c']]);
                    return [2 /*return*/];
            }
        });
    }); });
    test('Can recover from a failed task with orElse', function () {
        var resolve = jest.fn();
        var reject = jest.fn();
        var task = index_1.default.rejected('a');
        task.orElse(function (v) { return index_1.default.of(v.toUpperCase()); }).fork(resolve, reject);
        expect(resolve).toHaveBeenCalledWith('A');
        expect(reject).not.toHaveBeenCalled();
    });
    test('Can concat a task', function () { return __awaiter(_this, void 0, void 0, function () {
        var task1, task2, task, result;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    task1 = new index_1.default(function (resolve, _) {
                        setTimeout(function () {
                            resolve('a');
                        }, 60);
                    });
                    task2 = new index_1.default(function (resolve, _) {
                        setTimeout(function () {
                            resolve('b');
                        }, 40);
                    });
                    task = task1.concat(task2);
                    return [4 /*yield*/, task.toPromise()];
                case 1:
                    result = _a.sent();
                    expect(result).toBe('b');
                    return [2 /*return*/];
            }
        });
    }); });
    test('Can race tasks', function () { return __awaiter(_this, void 0, void 0, function () {
        var task1, task2, task3, result;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    task1 = new index_1.default(function (resolve, _) {
                        setTimeout(function () {
                            resolve('a');
                        }, 60);
                    });
                    task2 = new index_1.default(function (resolve, _) {
                        setTimeout(function () {
                            resolve('b');
                        }, 40);
                    });
                    task3 = new index_1.default(function (resolve, _) {
                        setTimeout(function () {
                            resolve('c');
                        }, 20);
                    });
                    return [4 /*yield*/, index_1.default.race([task1, task2, task3]).toPromise()];
                case 1:
                    result = _a.sent();
                    expect(result).toBe('c');
                    return [2 /*return*/];
            }
        });
    }); });
    test('Can create parallel tasks', function () { return __awaiter(_this, void 0, void 0, function () {
        var task1, task2, task3, result;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    task1 = new index_1.default(function (resolve, _) {
                        setTimeout(function () {
                            resolve('a');
                        }, 60);
                    });
                    task2 = new index_1.default(function (resolve, _) {
                        setTimeout(function () {
                            resolve('b');
                        }, 40);
                    });
                    task3 = new index_1.default(function (resolve, _) {
                        setTimeout(function () {
                            resolve('c');
                        }, 20);
                    });
                    return [4 /*yield*/, index_1.default.parallel([task1, task2, task3]).toPromise()];
                case 1:
                    result = _a.sent();
                    expect(result).toEqual(['a', 'b', 'c']);
                    return [2 /*return*/];
            }
        });
    }); });
});
