declare type TaskComputation<A, B> = (resolve: (value?: A) => void, reject: (reason?: B) => void) => void;
declare type SuccessCallback<A> = (a: A) => any;
declare type ErrorCallback<B> = (b: B) => any;
declare type PromiseCreator<A> = () => Promise<A>;
export default class Task<A> {
    static empty(): Task<{}>;
    static of<A>(value: A): Task<A>;
    static rejected(reason?: any): Task<any>;
    static race<A>(tasks: Array<Task<A>>): Task<A>;
    static parallel(tasks: Array<Task<any>>): Task<any[]>;
    static fromPromise<A>(promise: Promise<A> | PromiseCreator<A>): Task<A>;
    computation: TaskComputation<A, any>;
    constructor(computation: TaskComputation<A, any>);
    fork(success: SuccessCallback<A>, error?: ErrorCallback<any>): void;
    map<C>(fn: (a: A) => C): Task<C>;
    mapRejected<C>(fn: (a: A) => C): Task<A>;
    chain<C>(fn: (a: A) => Task<C>): Task<C>;
    orElse<C>(fn: (a: A) => Task<C>): Task<C>;
    concat(that: Task<A>): Task<A>;
    toPromise(): Promise<A>;
}
export {};
