"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var noop = function () { return undefined; };
var Task = /** @class */ (function () {
    function Task(computation) {
        this.computation = computation;
    }
    Task.empty = function () {
        return new Task(noop);
    };
    Task.of = function (value) {
        return new Task(function (resolve) {
            resolve(value);
        });
    };
    Task.rejected = function (reason) {
        return new Task(function (_, reject) {
            reject(reason);
        });
    };
    Task.race = function (tasks) {
        return new Task(function (resolve, reject) {
            var done = false;
            var guard = function (fn) { return function (x) {
                if (!done) {
                    done = true;
                    return fn(x);
                }
            }; };
            tasks.forEach(function (task) {
                task.fork(guard(resolve), guard(reject));
            });
        });
    };
    Task.parallel = function (tasks) {
        return new Task(function (resolve, reject) {
            var length = tasks.length;
            var results = new Array(length);
            var resolved = [];
            tasks.forEach(function (task, idx) {
                task.fork(function (result) {
                    results[idx] = result;
                    if (resolved.push(task) === length) {
                        resolve(results);
                    }
                }, function (err) {
                    reject(err);
                });
            });
        });
    };
    Task.fromPromise = function (promise) {
        return new Task(function (resolve, reject) {
            ;
            (typeof promise === 'function' ? promise() : promise).then(resolve, reject);
        });
    };
    Task.prototype.fork = function (success, error) {
        if (error === void 0) { error = noop; }
        this.computation(success, error);
    };
    Task.prototype.map = function (fn) {
        var _this = this;
        return new Task(function (resolve, reject) {
            return _this.fork(function (a) { return resolve(fn(a)); }, function (b) { return reject(b); });
        });
    };
    Task.prototype.mapRejected = function (fn) {
        var _this = this;
        return new Task(function (resolve, reject) {
            return _this.fork(function (a) { return resolve(a); }, function (b) { return reject(fn(b)); });
        });
    };
    Task.prototype.chain = function (fn) {
        var _this = this;
        return new Task(function (resolve, reject) {
            return _this.fork(function (a) { return fn(a).fork(resolve, reject); }, function (b) { return reject(b); });
        });
    };
    Task.prototype.orElse = function (fn) {
        var _this = this;
        return new Task(function (resolve, reject) {
            return _this.fork(function (a) { return fn(a).fork(resolve, reject); }, function (b) { return fn(b).fork(resolve, reject); });
        });
    };
    Task.prototype.concat = function (that) {
        return Task.race([this, that]);
    };
    Task.prototype.toPromise = function () {
        var _this = this;
        return new Promise(function (resolve, reject) { return _this.fork(resolve, reject); });
    };
    return Task;
}());
exports.default = Task;
